/*
Linear Differential Evolution (L-DE), not depending on the coordinate system.
In particular, it is almost invariant by rotation
Note: complete invariance is impossible when the search space is not a hypersphere
Here, it is as always a hyperrectangle, but you can normalise it in problemDef().

 Contact for remarks, suggestions etc.:
Maurice.Clerc@WriteLe.com 

Last update (which is also the version code):
 2011-11-04
	
*/

#include "main.h"

// =================================================
int main () 
{ 
	struct position bestBest; // Best position over all runs
	int d;			// Current dimension
	double D;
	double error;			// Current error
	double errorMean;		// Average error
	double errorMin;		// Best result over all runs
	double errorMeanBest[R_max]; 
	double evalMean;		// Mean number of evaluations
	int functionCode;
	int func[funcMax]; // List of functions (codes) to optimise
	int indFunc;

	int nbFunc;
	int nFailure;		// Number of unsuccessful runs
	struct param param;
	struct problem pb; 

	int run, runMax; 
	struct result result; 
	
	double Smean;
	double success[funcMax];
	double successRate;
	int t;
	double variance;
	double zz;

	E = exp ((long double) 1);  
	pi = acos ((long double) -1); 
	errMax=0;
	
	// Files
	f_run = fopen ("f_run.txt", "w");  // Save the result of each run
	f_synth = fopen ("f_synth.txt", "w"); // Save the averaged results


	//------------------------------------------------ PARAMETERS
	
			// Bells and Whistles
				// May improve the performance (not always)
				// Often just rules of thumbs
	param.BW[0]=0; 	//	0 => same swarm size for each run 	
									//	1 => random swarm size around the given mean
									
	param.BW[1]=1; 	//	0 =>  "standard" method	 (i.e. dimension by dimension)
									//*  1 => rotation invariant method ("geometrical")
	//--------
	
	param.confin=0; 	// 0 => keep inside the search space (supposed to be a D-rectangle)
										// 1 => no confinement 
										//   WARNING: may be very slow (and bad) for discrete problems

	//param.mean=0.5; //Default: 0.5. For some functions 0 is better, though 
	//param.sigma=0.35; // Default: 1./12 (standard deviation of U(0,1))			

switch(param.BW[1])
{
	default: // Standard DE
	Smean=80; //Swarm size or Mean swarm size (if BW[0]=1). Suggested: 80 
	param.C=0.9;// 0.5
	param.F= 0.8; 
	break;
	
	case 0: // LS-DE
					// Note: the "good" parameter values are not necessarily the same as 
					//        for standard DE
	Smean=80; 
	param.C=0.9; 
	param.F=0.8;
	break;

}

	// ----------------------------------------------- PROBLEM
		runMax = 10; // Numbers of runs (for each function)
		
	// Functions to optimise
	nbFunc=1; // Number of functions  (14)
	func[0]=102; // 4
	func[1]=15;  // 15 
	func[2]=17;  // 17
	func[3]=18; //18
	
	func[4]=20;
	func[5]=21;
	func[6]=100;
	func[7]=10010; 
	
	func[8]=102; 
	func[9]=103;
	func[10]=104;
	func[11]=105;	
	
	func[12]=106;	
	func[13]=10610;	
				
	/* (see problemDef( ) for precise definitions)
	0 Parabola (Sphere)
	1 Griewank
	2 Rosenbrock (Banana)
	3 Rastrigin
	4 Tripod (dimension 2) // Mini benchmark
	5 Ackley
	6 Schwefel
	7 Schwefel 1.2
	8 Schwefel 2.22
	9 Neumaier 3
	10 G3
	11 Network optimisation (Warning: see problemDef() and also perf() for
			problem elements (number of BTS and BSC)
	12 Schwefel
	13 2D Goldstein-Price
	14 Schaffer f6
	15 Step	
	16 Schwefel 2.21
	17 Lennard-Jones
	18 Gear train		// Mini benchmark
	19 Sine_sine function
	20 Perm function
	21 Compression Spring	// Mini benchmark
	22 Cellular phone (2D) 
	23 Penalized
	24 2D repulsion
	25 Pressure Vessel (penalty method)
	

	CEC 2005 benchmark  (no more than 30D. See cec2005data.c)
	100 F1 (shifted Parabola/Sphere)  30D
	10010   "  10D
	102 F6 (shifted Rosenbrock)  // Mini benchmark
	103 F9 (shifted Rastrigin) 
	104 F2 Schwefel 
	105 F7 Griewank  (NOT rotated)
	106 F8 Ackley  (NOT rotated) 
	10610  " 10D

*/ 

	if (runMax > R_max) runMax = R_max;

	for(indFunc=0;indFunc<nbFunc;indFunc++) // Loop on problems
	{
		functionCode =func[indFunc];

		// Some information
		printf ("\n Function %i ", functionCode);

		// Define the problem
		pb=problemDef(functionCode);			

		// ----------------------------------------------- RUNS	
		errorMean = 0;	    
		evalMean = 0;	    
		nFailure = 0;	
		D=pb.SS.D;

		seed_rand_kiss(1294404794); // Initialise the RNG for reproducible results

// "Warm up" the RNG
		for (t=0;t<10000;t++) 
		{
		zz=alea(0,1); 
		}
		
		for (run = 0; run < runMax; run++)  
		{
			if(param.BW[0]==0) param.S=Smean; // Constant swarm size
			else // Random swarm size "around" the mean value
			param.S=(int)(0.5*(0.5+alea(Smean-D/2,Smean+D/2)+alea(Smean-D/2,Smean+D/2))); 

			result = DE(param, pb);
			error = result.error;

			if (error > pb.epsilon) // Failure
				nFailure = nFailure + 1;

			if(pb.SS.normalise>0)
			{
				for(d=0;d<pb.SS.D;d++)
					result.SW.X[result.SW.best].x[d]=
					pb.SS.min[d]+(pb.SS.max[d]-pb.SS.min[d])*result.SW.X[result.SW.best].x[d]
					/pb.SS.normalise;
			}		

			// Memorize the best (useful if more than one run)
			if(run==0) bestBest=result.SW.X[result.SW.best];
			else
				if(error<bestBest.f) bestBest=result.SW.X[result.SW.best];

			// Result display
			errorMean=errorMean+error;
			printf ("\nRun %i. S %i,  Eval %f. Error %e ", run+1, param.S, result.nEval, error);
			printf(" Mean %e",errorMean/(run+1));
			printf("  Success  %.2f%%",100*(1-(double)nFailure/(run+1)));

			// Best position display
			//for (d=0;d<pb.SS.D;d++) printf(" %f",result.SW.P[result.SW.best].x[d]);

			// Save result
					fprintf( f_run, "\n%i %.0f %e %e ", run+1, result.nEval,  error, errorMean/(run+1) );
			// Save best position
			//		for ( d = 0; d < pb.SS.D; d++ ) fprintf( f_run, " %f",  result.SW.P[result.SW.best].x[d] );

			// Compute/save some statistical information
			if (run == 0)
				errorMin = error;
			else if (error < errorMin)
				errorMin = error;
				
			evalMean = evalMean + result.nEval;	
			errorMeanBest[run] = error;
		}		// End loop on "run"

		// ---------------------END 
		
		// Display some statistical information
		evalMean = evalMean / (double) runMax;   
		errorMean = errorMean / (double) runMax;

		printf ("\n Eval. (mean)= %f", evalMean);	
		printf ("\n Error (mean) = %e", errorMean);

		// Variance
		variance = 0;

		for (run = 0; run < runMax; run++)
			variance = variance + pow (errorMeanBest[run] - errorMean, 2);

		variance = sqrt (variance / runMax);	    
		printf ("\n Std. dev. %f", variance); 

		// Success rate and minimum value
		printf("\n Failure(s) %i  ",nFailure);

		successRate = 100 * (1 - nFailure / (double) runMax);			
		printf ("Success rate = %.2f%%", successRate);
		success[indFunc]=successRate;

		printf ("\n Best min value = %1.20e", errorMin);
		printf ("\nPosition of the optimum: ");
		for (d=0;d<pb.SS.D;d++) printf(" %f",bestBest.x[d]);
		

		// Save	
		fprintf (f_synth, "%i %i %.0f %e %e %f %f", functionCode, pb.SS.D,successRate,
		    errorMean, variance, evalMean,bestBest.f);
		for (d=0;d<pb.SS.D;d++) fprintf(f_synth, " %1.20e",bestBest.x[d]);  
		fprintf(f_synth,"\n"); 

// Specific save for Repulsive problem
if(pb.function==24)
{		
	for (d=0;d<pb.SS.D-1;d=d+2) 
	{
	fprintf(f_synth, " %1.20e %1.20e",bestBest.x[d],bestBest.x[d+1]);  
	fprintf(f_synth,"\n"); 
	}	
}	     	   

} // End "for ind[..."

printf("\n errMax : %f",errMax);
		// Repeat informations
	printf("\n---------");
	printf("\n Function(s):");
		for(indFunc=0;indFunc<nbFunc;indFunc++) // Loop on problems
		{
			functionCode =func[indFunc];
			printf(" %i",functionCode);
		}
		printf("\n Confinement: "); 
		if(param.confin==0) printf("YES"); else printf("NO");

		printf("\n BW = (%i, %i)",param.BW[0],param.BW[1]);
		printf("\n Swarm size: ");
		if(param.BW[0]==0) printf("%i",(int)Smean); else printf(" mean %i",(int)Smean);
		printf("\n F = %f",param.F); 
	printf("\n C = %f", param.C);


	return 0; // End of main program
}
// ===============================================================
#include "alea.c"
#include "perf.c"
#include "problemDef.c"
#include "DE.c"
#include "tools.c"


